/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "waverecorder.h" 
#include "string.h"
/** @addtogroup STM32F4-Discovery_Audio_Player_Recorder
* @{
*/ 

/* Private typedef -----------------------------------------------------------*/
typedef struct {
  int32_t offset;
  uint32_t fptr;
}Audio_BufferTypeDef;

/* Private define ------------------------------------------------------------*/


/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/

/* Wave recorded counter.*/
__IO uint32_t WaveCounter1 = 0;
__IO uint32_t WaveCounter2 = 0;

extern __IO uint32_t CmdIndex, LEDsState, TimeRecBase;

FIL WavFile1, WavFile2;
/* USB variable to check if USB connected or not*/
extern MSC_ApplicationTypeDef AppliState;

/* Variable used to switch play from audio sample available on USB to recorded file*/
uint32_t WaveRecStatus1 = 0; 
uint32_t WaveRecStatus2 = 0; 

uint8_t pHeaderBuff1[44];
uint16_t WrBuffer1[WR_BUFFER_SIZE];

uint8_t pHeaderBuff2[44];
uint16_t WrBuffer2[WR_BUFFER_SIZE];

static uint16_t RecBuf1[PCM_OUT_SIZE*2];/* PCM stereo samples are saved in RecBuf */
static uint16_t InternalBuffer1[INTERNAL_BUFF_SIZE];
__IO uint32_t ITCounter1 = 0;

static uint16_t RecBuf2[PCM_OUT_SIZE*2];/* PCM stereo samples are saved in RecBuf */
static uint16_t InternalBuffer2[INTERNAL_BUFF_SIZE];
__IO uint32_t ITCounter2 = 0;

Audio_BufferTypeDef  BufferCtl1, BufferCtl2;

/* Temporary data sample */
__IO uint32_t AUDIODataReady = 0, AUDIOBuffOffset = 0;

WAVE_FormatTypeDef WaveFormat;

__IO FRESULT res;
/* Private function prototypes -----------------------------------------------*/

/* Private functions ---------------------------------------------------------*/
static uint32_t WavProcess_EncInit(uint32_t Freq, uint8_t* pHeader);
static uint32_t WavProcess_HeaderInit(uint8_t* pHeader, WAVE_FormatTypeDef* pWaveFormatStruct);
static uint32_t WavProcess_HeaderUpdate1(uint8_t* pHeader, WAVE_FormatTypeDef* pWaveFormatStruct);
static uint32_t WavProcess_HeaderUpdate2(uint8_t* pHeader, WAVE_FormatTypeDef* pWaveFormatStruct);


/**
  * @brief  Start Audio recording.
  * @param  pbuf: pointer to a buffer
  *         size: Buffer size
  * @retval None
  */
uint8_t WaveRecorderStart(uint16_t* pBuf1, uint16_t* pBuf2, uint32_t wSize)
{
  return (BSP_AUDIO_IN_Record(pBuf1, pBuf2, wSize));
}

/**
  * @brief  Stop Audio recording.
  * @param  None
  * @retval None
  */
uint32_t WaveRecorderStop(void)
{
  return BSP_AUDIO_IN_Stop();
}

/**
  * @brief  Update the recorded data. 
  * @param  None
  * @retval None
  */
void WaveRecorderProcess(void)
{     
  /* Current size of the recorded buffer */
  uint32_t byteswritten1 = 0;
  uint32_t byteswritten2 = 0;	

  WaveCounter1 = 0;
	WaveCounter2 = 0;
  LEDsState = LEDS_OFF;
	
  /* Remove Wave file if it exists on USB Flash Disk */
  f_unlink(REC_WAVE_NAME1);
  f_unlink(REC_WAVE_NAME2);
  /* Open the file to write on it */
  if((AppliState == APPLICATION_IDLE) || (f_open(&WavFile1, REC_WAVE_NAME1, FA_CREATE_ALWAYS | FA_WRITE) != FR_OK) || (f_open(&WavFile2, REC_WAVE_NAME2, FA_CREATE_ALWAYS | FA_WRITE) != FR_OK))
  {
		while(1)
		{
			/* Toggle LED5 in infinite loop to signal that: USB Flash Disk is not connected/removed
			or an issue has occurred when creating/opening Wave file */
			BSP_LED_Toggle(LED5); 
		}
  }
  else
  {
    WaveRecStatus1 = 1;
    WaveRecStatus2 = 1;
  }
  /* Initialize header file */
  WavProcess_EncInit(DEFAULT_AUDIO_IN_FREQ, pHeaderBuff1);
  WavProcess_EncInit(DEFAULT_AUDIO_IN_FREQ, pHeaderBuff2);
  
  /* Write the header Wave */
  f_write(&WavFile1, pHeaderBuff1, 44, (void *)&byteswritten1);
  f_write(&WavFile2, pHeaderBuff2, 44, (void *)&byteswritten2);
  
  /* Increment the Wave counter */  
  BufferCtl1.fptr = byteswritten1;
	BufferCtl2.fptr = byteswritten2;
	BufferCtl1.offset = BUFFER_OFFSET_NONE;
	BufferCtl2.offset = BUFFER_OFFSET_NONE;
  BSP_AUDIO_IN_Init(DEFAULT_AUDIO_IN_FREQ, DEFAULT_AUDIO_IN_BIT_RESOLUTION, DEFAULT_AUDIO_IN_CHANNEL_NBR);
  BSP_AUDIO_IN_Record((uint16_t*)&InternalBuffer1[0], (uint16_t*)&InternalBuffer2[0], INTERNAL_BUFF_SIZE);
  /* Reset the time recording base variable */
  
	TimeRecBase = 0;
  ITCounter1 = 0;
  ITCounter2 = 0;
  LEDsState = LED3_TOGGLE;
	int i = 0;
  while(AppliState != APPLICATION_IDLE)
  { 
    /* Wait for the recording time */  
    if(TimeRecBase <= DEFAULT_TIME_REC)
    {
			if(HAL_GPIO_ReadPin(I2S2_MOSI_GPIO_PORT, I2S2_MOSI_PIN) == GPIO_PIN_RESET)
			{
				//InternalBuffer1[i] = 0;
				WrBuffer1[i] = 0;
				BSP_LED_On(LED4);
			}
			else
			{
				BSP_LED_On(LED5);
				//InternalBuffer1[i] = 1;
				WrBuffer1[i] = 32836;
			}
			
			if(HAL_GPIO_ReadPin(I2S3_MOSI_GPIO_PORT, I2S3_MOSI_PIN) == GPIO_PIN_RESET)
			{
				//InternalBuffer2[i] = 0;
				BSP_LED_On(LED4);
				WrBuffer2[i] = 0;
			}
			else
			{
				BSP_LED_On(LED5);
				WrBuffer2[i] = 32836;
			}
			/*if(i == INTERNAL_BUFF_SIZE/2 - 1)
			{
				BSP_AUDIO_IN_PDMToPCM((uint16_t*)&InternalBuffer1[0], (uint16_t*)&RecBuf1[0]);
				BSP_AUDIO_IN_PDMToPCM((uint16_t*)&InternalBuffer2[0], (uint16_t*)&RecBuf2[0]);
				memcpy((uint16_t*)&WrBuffer1[0], RecBuf1, PCM_OUT_SIZE*4);
				memcpy((uint16_t*)&WrBuffer2[0], RecBuf2, PCM_OUT_SIZE*4);
			}*/
			if(i == INTERNAL_BUFF_SIZE - 1)
			{
				/*BSP_AUDIO_IN_PDMToPCM((uint16_t*)&InternalBuffer1[INTERNAL_BUFF_SIZE/2], (uint16_t*)&RecBuf1[0]);
				BSP_AUDIO_IN_PDMToPCM((uint16_t*)&InternalBuffer2[INTERNAL_BUFF_SIZE/2], (uint16_t*)&RecBuf2[0]);
				memcpy((uint16_t*)&WrBuffer1[PCM_OUT_SIZE*2], RecBuf1, PCM_OUT_SIZE*4);
				memcpy((uint16_t*)&WrBuffer2[PCM_OUT_SIZE*2], RecBuf2, PCM_OUT_SIZE*4);*/
				AUDIODataReady = 1;
				i = 0;
				BSP_LED_On(LED6);
			}
      /* Check if there are Data to write in Usb Key */
      if(AUDIODataReady == 1)
      {
				//AUDIOBuffOffset = WR_BUFFER_SIZE/2;
        /* write buffer in file */
        res = f_write(&WavFile1, (uint8_t*)(WrBuffer1+AUDIOBuffOffset), WR_BUFFER_SIZE, (void*)&byteswritten1);
        if(res != FR_OK)
        {
          Error_Handler();
        }
        res = f_write(&WavFile2, (uint8_t*)(WrBuffer2+AUDIOBuffOffset), WR_BUFFER_SIZE, (void*)&byteswritten2);
        if(res != FR_OK)
        {
          Error_Handler();
        }
        BufferCtl1.fptr += byteswritten1;
				BufferCtl2.fptr += byteswritten2;
        AUDIODataReady = 0;
      }
      
      /* User button pressed */
      if(CmdIndex != CMD_RECORD)
      {
        /* Stop Audio Recording */
        WaveRecorderStop();
        /* Switch Command Index to Play */
        //CmdIndex = CMD_PLAY;
        /* Toggoling LED6 to signal Play */
        LEDsState = LED6_TOGGLE;
        break;
      }
			i++;
    }
    else /* End of recording time DEFAULT_TIME_REC */
    {
      /* Stop Audio Recording */
      WaveRecorderStop();
      /* Change Command Index to Stop */
      CmdIndex = CMD_STOP;
      /* Toggoling LED4 to signal Stop */
      LEDsState = LED4_TOGGLE;
      AUDIODataReady = 0;
      break;
    }
  }
  f_lseek(&WavFile1, 0);
  f_lseek(&WavFile2, 0);
  
  /* Parse the wav file header and extract required information */
  WavProcess_HeaderUpdate1(pHeaderBuff1, &WaveFormat);
  f_write(&WavFile1, pHeaderBuff1, 44, (void*)&byteswritten1);
  WavProcess_HeaderUpdate2(pHeaderBuff2, &WaveFormat);
  f_write(&WavFile2, pHeaderBuff2, 44, (void*)&byteswritten2);
  
  /* Close file and unmount MyFilesystem */
  f_close (&WavFile1);
  f_close (&WavFile2);
  f_mount(NULL, 0, 1);
  
  /* Change Command Index to Play */
  //CmdIndex = CMD_PLAY;
}

/**
  * @brief Calculates the remaining file size and new position of the pointer.
  * @param None
  * @retval None
  */
void BSP_AUDIO_IN_TransferComplete_CallBack(void)
{
	int i;
	for(i = INTERNAL_BUFF_SIZE/2; i < INTERNAL_BUFF_SIZE; i++)
	{
		if(HAL_GPIO_ReadPin(I2S2_MOSI_GPIO_PORT, I2S2_MOSI_PIN) == GPIO_PIN_RESET)
		{
			InternalBuffer1[i] = 0;
			BSP_LED_On(LED4);
		}
		else
		{
			BSP_LED_On(LED5);
			InternalBuffer1[i] = 1;
		}
	}
  /* PDM to PCM data convert */
  BSP_AUDIO_IN_PDMToPCM((uint16_t*)&InternalBuffer1[INTERNAL_BUFF_SIZE/2], (uint16_t*)&RecBuf1[0]);
  BSP_AUDIO_IN_PDMToPCM((uint16_t*)&InternalBuffer2[INTERNAL_BUFF_SIZE/2], (uint16_t*)&RecBuf2[0]);
  
  /* Copy PCM data in internal buffer */
  memcpy((uint16_t*)&WrBuffer1[ITCounter1 * (PCM_OUT_SIZE*2)], RecBuf1, PCM_OUT_SIZE*4);
  memcpy((uint16_t*)&WrBuffer2[ITCounter2 * (PCM_OUT_SIZE*2)], RecBuf2, PCM_OUT_SIZE*4);
  
  BufferCtl1.offset = BUFFER_OFFSET_NONE;
	BufferCtl2.offset = BUFFER_OFFSET_NONE;
  
  if(ITCounter1 == (WR_BUFFER_SIZE/(PCM_OUT_SIZE*4))-1)
  {
    AUDIODataReady = 1;
    AUDIOBuffOffset = 0;
    ITCounter1++;
  }
  else if(ITCounter1 == (WR_BUFFER_SIZE/(PCM_OUT_SIZE*2))-1)
  {
    AUDIODataReady = 1;
    AUDIOBuffOffset = WR_BUFFER_SIZE/2;
    ITCounter1 = 0;
  }
  else
  {
    ITCounter1++;
  }

  if(ITCounter2 == (WR_BUFFER_SIZE/(PCM_OUT_SIZE*4))-1)
  {
    AUDIODataReady = 1;
    AUDIOBuffOffset = 0;
    ITCounter2++;
  }
  else if(ITCounter1 == (WR_BUFFER_SIZE/(PCM_OUT_SIZE*2))-1)
  {
    AUDIODataReady = 1;
    AUDIOBuffOffset = WR_BUFFER_SIZE/2;
    ITCounter2 = 0;
  }
  else
  {
    ITCounter2++;
  }
	//AUDIODataReady = 1;
}

/**
  * @brief  Manages the DMA Half Transfer complete interrupt.
  * @param  None
  * @retval None
  */
void BSP_AUDIO_IN_HalfTransfer_CallBack(void)
{
	int i;
	for(i = 0; i < INTERNAL_BUFF_SIZE/2; i++)
	{
		if(HAL_GPIO_ReadPin(I2S2_MOSI_GPIO_PORT, I2S2_MOSI_PIN) == GPIO_PIN_RESET)
		{
			InternalBuffer1[i] = 0;
			BSP_LED_On(LED4);
		}
		else
		{
			BSP_LED_On(LED5);
			InternalBuffer1[i] = 1;
		}
	}
  /* PDM to PCM data convert */
  BSP_AUDIO_IN_PDMToPCM((uint16_t*)&InternalBuffer1[0], (uint16_t*)&RecBuf1[0]);
  BSP_AUDIO_IN_PDMToPCM((uint16_t*)&InternalBuffer2[0], (uint16_t*)&RecBuf2[0]);
  /* Copy PCM data in internal buffer */
  memcpy((uint16_t*)&WrBuffer1[ITCounter1 * (PCM_OUT_SIZE*2)], RecBuf1, PCM_OUT_SIZE*4);
  memcpy((uint16_t*)&WrBuffer2[ITCounter2 * (PCM_OUT_SIZE*2)], RecBuf2, PCM_OUT_SIZE*4);
  
  BufferCtl1.offset = BUFFER_OFFSET_NONE;
	BufferCtl2.offset = BUFFER_OFFSET_NONE;
  
  if(ITCounter1 == (WR_BUFFER_SIZE/(PCM_OUT_SIZE*4))-1)
  {
    AUDIODataReady = 1;
    AUDIOBuffOffset = 0;
    ITCounter1++;
  }
  else if(ITCounter1 == (WR_BUFFER_SIZE/(PCM_OUT_SIZE*2))-1)
  {
    AUDIODataReady = 1;
    AUDIOBuffOffset = WR_BUFFER_SIZE/2;
    ITCounter1 = 0;
  }
  else
  {
    ITCounter1++;
  }

  if(ITCounter2 == (WR_BUFFER_SIZE/(PCM_OUT_SIZE*4))-1)
  {
    AUDIODataReady = 1;
    AUDIOBuffOffset = 0;
    ITCounter2++;
  }
  else if(ITCounter2 == (WR_BUFFER_SIZE/(PCM_OUT_SIZE*2))-1)
  {
    AUDIODataReady = 1;
    AUDIOBuffOffset = WR_BUFFER_SIZE/2;
    ITCounter2 = 0;
  }
  else
  {
    ITCounter2++;
  }
	//AUDIODataReady = 1;
}

/**
  * @brief  Encoder initialization.
  * @param  Freq: Sampling frequency.
  * @param  pHeader: Pointer to the WAV file header to be written.  
  * @retval 0 if success, !0 else.
  */
static uint32_t WavProcess_EncInit(uint32_t Freq, uint8_t* pHeader)
{
  /* Initialize the encoder structure */
  WaveFormat.SampleRate = Freq;        /* Audio sampling frequency */
  WaveFormat.NbrChannels = 2;          /* Number of channels: 1:Mono or 2:Stereo */
  WaveFormat.BitPerSample = 16;        /* Number of bits per sample (16, 24 or 32) */
  WaveFormat.FileSize = 0x001D4C00;    /* Total length of useful audio data (payload) */
  WaveFormat.SubChunk1Size = 44;       /* The file header chunk size */
  WaveFormat.ByteRate = (WaveFormat.SampleRate * \
                        (WaveFormat.BitPerSample/8) * \
                         WaveFormat.NbrChannels);            /* Number of bytes per second  (sample rate * block align)  */
  WaveFormat.BlockAlign = WaveFormat.NbrChannels * \
                         (WaveFormat.BitPerSample/8);      /* channels * bits/sample / 8 */
  
  /* Parse the wav file header and extract required information */
  if(WavProcess_HeaderInit(pHeader, &WaveFormat))
  {
    return 1;
  }
  if(WavProcess_HeaderInit(pHeader, &WaveFormat))
  {
    return 1;
  }
  return 0;
}

/**
  * @brief  Initialize the wave header file
  * @param  pHeader: Header Buffer to be filled
  * @param  pWaveFormatStruct: Pointer to the wave structure to be filled.
  * @retval 0 if passed, !0 if failed.
  */
static uint32_t WavProcess_HeaderInit(uint8_t* pHeader, WAVE_FormatTypeDef* pWaveFormatStruct)
{
  /* write chunkID, must be 'RIFF'  ------------------------------------------*/
  pHeader[0] = 'R';
  pHeader[1] = 'I';
  pHeader[2] = 'F';
  pHeader[3] = 'F';
  
  /* Write the file length ---------------------------------------------------*/
  /* The sampling time: this value will be be written back at the end of the 
  recording opearation.  Example: 661500 Btyes = 0x000A17FC, byte[7]=0x00, byte[4]=0xFC */
  pHeader[4] = 0x00;
  pHeader[5] = 0x4C;
  pHeader[6] = 0x1D;
  pHeader[7] = 0x00;
  /* Write the file format, must be 'WAVE' -----------------------------------*/
  pHeader[8]  = 'W';
  pHeader[9]  = 'A';
  pHeader[10] = 'V';
  pHeader[11] = 'E';
  
  /* Write the format chunk, must be'fmt ' -----------------------------------*/
  pHeader[12]  = 'f';
  pHeader[13]  = 'm';
  pHeader[14]  = 't';
  pHeader[15]  = ' ';
  
  /* Write the length of the 'fmt' data, must be 0x10 ------------------------*/
  pHeader[16]  = 0x10;
  pHeader[17]  = 0x00;
  pHeader[18]  = 0x00;
  pHeader[19]  = 0x00;
  
  /* Write the audio format, must be 0x01 (PCM) ------------------------------*/
  pHeader[20]  = 0x01;
  pHeader[21]  = 0x00;
  
  /* Write the number of channels, ie. 0x01 (Mono) ---------------------------*/
  pHeader[22]  = pWaveFormatStruct->NbrChannels;
  pHeader[23]  = 0x00;
  
  /* Write the Sample Rate in Hz ---------------------------------------------*/
  /* Write Little Endian ie. 8000 = 0x00001F40 => byte[24]=0x40, byte[27]=0x00*/
  pHeader[24]  = (uint8_t)((pWaveFormatStruct->SampleRate & 0xFF));
  pHeader[25]  = (uint8_t)((pWaveFormatStruct->SampleRate >> 8) & 0xFF);
  pHeader[26]  = (uint8_t)((pWaveFormatStruct->SampleRate >> 16) & 0xFF);
  pHeader[27]  = (uint8_t)((pWaveFormatStruct->SampleRate >> 24) & 0xFF);
  
  /* Write the Byte Rate -----------------------------------------------------*/
  pHeader[28]  = (uint8_t)((pWaveFormatStruct->ByteRate & 0xFF));
  pHeader[29]  = (uint8_t)((pWaveFormatStruct->ByteRate >> 8) & 0xFF);
  pHeader[30]  = (uint8_t)((pWaveFormatStruct->ByteRate >> 16) & 0xFF);
  pHeader[31]  = (uint8_t)((pWaveFormatStruct->ByteRate >> 24) & 0xFF);
  
  /* Write the block alignment -----------------------------------------------*/
  pHeader[32]  = pWaveFormatStruct->BlockAlign;
  pHeader[33]  = 0x00;
  
  /* Write the number of bits per sample -------------------------------------*/
  pHeader[34]  = pWaveFormatStruct->BitPerSample;
  pHeader[35]  = 0x00;
  
  /* Write the Data chunk, must be 'data' ------------------------------------*/
  pHeader[36]  = 'd';
  pHeader[37]  = 'a';
  pHeader[38]  = 't';
  pHeader[39]  = 'a';
  
  /* Write the number of sample data -----------------------------------------*/
  /* This variable will be written back at the end of the recording operation */
  pHeader[40]  = 0x00;
  pHeader[41]  = 0x4C;
  pHeader[42]  = 0x1D;
  pHeader[43]  = 0x00;
  
  /* Return 0 if all operations are OK */
  return 0;
}

/**
  * @brief  Initialize the wave header file
  * @param  pHeader: Header Buffer to be filled
  * @param  pWaveFormatStruct: Pointer to the wave structure to be filled.
  * @retval 0 if passed, !0 if failed.
  */
static uint32_t WavProcess_HeaderUpdate1(uint8_t* pHeader, WAVE_FormatTypeDef* pWaveFormatStruct)
{
  /* Write the file length ----------------------------------------------------*/
  /* The sampling time: this value will be be written back at the end of the 
  recording opearation.  Example: 661500 Btyes = 0x000A17FC, byte[7]=0x00, byte[4]=0xFC */
  pHeader[4] = (uint8_t)(BufferCtl1.fptr);
  pHeader[5] = (uint8_t)(BufferCtl1.fptr >> 8);
  pHeader[6] = (uint8_t)(BufferCtl1.fptr >> 16);
  pHeader[7] = (uint8_t)(BufferCtl1.fptr >> 24);
  /* Write the number of sample data -----------------------------------------*/
  /* This variable will be written back at the end of the recording operation */
  BufferCtl1.fptr -=44;
  pHeader[40] = (uint8_t)(BufferCtl1.fptr); 
  pHeader[41] = (uint8_t)(BufferCtl1.fptr >> 8);
  pHeader[42] = (uint8_t)(BufferCtl1.fptr >> 16);
  pHeader[43] = (uint8_t)(BufferCtl1.fptr >> 24); 
  /* Return 0 if all operations are OK */
  return 0;
}

static uint32_t WavProcess_HeaderUpdate2(uint8_t* pHeader, WAVE_FormatTypeDef* pWaveFormatStruct)
{
  /* Write the file length ----------------------------------------------------*/
  /* The sampling time: this value will be be written back at the end of the 
  recording opearation.  Example: 661500 Btyes = 0x000A17FC, byte[7]=0x00, byte[4]=0xFC */
  pHeader[4] = (uint8_t)(BufferCtl2.fptr);
  pHeader[5] = (uint8_t)(BufferCtl2.fptr >> 8);
  pHeader[6] = (uint8_t)(BufferCtl2.fptr >> 16);
  pHeader[7] = (uint8_t)(BufferCtl2.fptr >> 24);
  /* Write the number of sample data -----------------------------------------*/
  /* This variable will be written back at the end of the recording operation */
  BufferCtl2.fptr -=44;
  pHeader[40] = (uint8_t)(BufferCtl2.fptr); 
  pHeader[41] = (uint8_t)(BufferCtl2.fptr >> 8);
  pHeader[42] = (uint8_t)(BufferCtl2.fptr >> 16);
  pHeader[43] = (uint8_t)(BufferCtl2.fptr >> 24); 
  /* Return 0 if all operations are OK */
  return 0;
}
